# Copyright (c) 2015, 2016 OpenStack Foundation.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

SBUS_CMD_PREFIX = 'SBUS_CMD_'

SBUS_CMD_HALT = 'SBUS_CMD_HALT'
SBUS_CMD_EXECUTE = 'SBUS_CMD_EXECUTE'
SBUS_CMD_START_DAEMON = 'SBUS_CMD_START_DAEMON'
SBUS_CMD_STOP_DAEMON = 'SBUS_CMD_STOP_DAEMON'
SBUS_CMD_DAEMON_STATUS = 'SBUS_CMD_DAEMON_STATUS'
SBUS_CMD_STOP_DAEMONS = 'SBUS_CMD_STOP_DAEMONS'
SBUS_CMD_PING = 'SBUS_CMD_PING'
SBUS_CMD_CANCEL = 'SBUS_CMD_CANCEL'
SBUS_CMD_NOP = 'SBUS_CMD_NOP'
